# -------------------------------------------------------------------------
# Copyright (c) Switch Automation Pty Ltd. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
"""
Switch Guides Scaffolder into the Switch Automation
=========================================

Complete package for helping with the creation of Platform Guides in Switch Automation Platform.
"""
__all__ = [
]

__version__ = "0.1.7"
